@@ -12,7 +12,8 @@ from datetime import timedelta
 
 from loguru._file_sink import FileSink
 
-from elasticsearch import Elasticsearch, NotFoundError as ESNotFoundError, helpers as es_helpers
+from elasticsearch import Elasticsearch, ApiError as ESApiError, NotFoundError as ESNotFoundError, helpers as es_helpers
+from confluent_kafka import Producer as KFKProducer
 
 from .trace import get_trace_id
 from .base import Utils
@@ -70,6 +71,136 @@ class InterceptHandler(logging.Handler):
         )
 
 
+class _BaseSink:
+    """基础日志投递基类
+    """
+
+    def __init__(self, buffer_maxsize=0xffff):
+
+        self._buffer = queue.Queue(buffer_maxsize)
+        self._closed = False
+
+        self._task = Thread(target=self._do_task)
+        self._task.start()
+
+    def write(self, message):
+
+        try:
+
+            if message.record[r'thread'].id == self._task.ident:
+                return
+
+            log_extra = message.record[r'extra']
+
+            if r'trace_id' not in log_extra and (trace_id := get_trace_id()) is not None:
+                log_extra[r'trace_id'] = trace_id
+
+            self._buffer.put_nowait(message)
+
+        except queue.Full as _:
+
+            if message.record[r'level'].no > logging.INFO:
+                sys.stderr.write(str(message))
+
+        except Exception as err:
+
+            sys.stderr.write(f'{str(err)}\n')
+
+    def close(self):
+
+        self._closed = True
+        self._task.join(10)
+
+    def _do_task(self):
+
+        while not self._closed:
+
+            messages = []
+
+            try:
+
+                messages.append(self._buffer.get(block=True, timeout=1))
+
+                for _ in range(min(self._buffer.qsize(), 1000)):
+                    messages.append(self._buffer.get_nowait())
+
+                if messages:
+                    self._write_logs(messages)
+
+            except queue.Empty as _:
+
+                pass
+
+            except Exception as err:
+
+                sys.stderr.write(f'{str(err)}\n')
+
+    def _write_logs(self, logs):
+
+        raise NotImplementedError()
+
+
+class QueuedFileSink(_BaseSink, FileSink):
+    """日志文件队列
+    """
+
+    def __init__(self, path, *, buffer_maxsize=0xffff, **kwargs):
+
+        _BaseSink.__init__(self, buffer_maxsize)
+        FileSink.__init__(self, path, **kwargs)
+
+    def _write_logs(self, logs):
+
+        for _log in logs:
+            FileSink.write(self, _log)
+
+
+class KafkaSink(_BaseSink):
+    """Kafka日志投递
+    """
+
+    def __init__(self, servers, topic, *, buffer_maxsize=0xffff, **kwargs):
+
+        super().__init__(buffer_maxsize)
+
+        kwargs[r'bootstrap.servers'] = servers
+
+        self._producer = KFKProducer(kwargs)
+
+        self._topic = topic
+
+    def _write_logs(self, logs):
+
+        for _log in logs:
+            self._producer.produce(
+                self._topic,
+                Utils.json_encode(
+                    {
+                        r'extra': _log.record[r'extra'],
+                        r'process': {
+                            r'id': _log.record[r'process'].id,
+                            r'name': _log.record[r'process'].name,
+                        },
+                        r'thread': {
+                            r'id': _log.record[r'thread'].id,
+                            r'name': _log.record[r'thread'].name,
+                        },
+                        r'level': {
+                            r'no': _log.record[r'level'].no,
+                            r'name': _log.record[r'level'].name,
+                        },
+                        r'module': f"{_log.record[r'name']}:{_log.record[r'function']}:{_log.record[r'line']}",
+                        r'message': _log.record[r'message'],
+                        r'timestamp': int(_log.record[r'time'].timestamp() * 1000),
+                    }
+                ),
+            )
+
+            self._producer.poll(0)
+
+        self._producer.flush()
+
+
 class ElasticsearchDataStreamUtil:
 
     def __init__(
@@ -99,6 +230,10 @@ class ElasticsearchDataStreamUtil:
         except ESNotFoundError as _:
             self._create_lifecycle()
             self._create_index_template()
+        except ESApiError as err:
+            sys.stderr.write(f'{str(err.info)}\n')
+        except Exception as err:
+            sys.stderr.write(f'{str(err)}\n')
 
     def _create_lifecycle(self):
 
@@ -129,9 +264,13 @@ class ElasticsearchDataStreamUtil:
 
             self._elasticsearch.ilm.put_lifecycle(name=self._policy_name, policy=policy)
 
+        except ESApiError as err:
+
+            sys.stderr.write(f'{str(err.info)}\n')
+
         except Exception as err:
 
-            sys.stderr.write(f'{str(err.body)}\n')
+            sys.stderr.write(f'{str(err)}\n')
 
     def _create_index_template(self):
 
@@ -184,7 +323,6 @@ class ElasticsearchDataStreamUtil:
                     },
                     r'@timestamp': {
                         r'type': r'date',
-                        r'format': r'epoch_millis',
                     },
                 }
             }
@@ -213,12 +351,16 @@ class ElasticsearchDataStreamUtil:
                 data_stream={},
             )
 
+        except ESApiError as err:
+
+            sys.stderr.write(f'{str(err.info)}\n')
+
         except Exception as err:
 
-            sys.stderr.write(f'{str(err.body)}\n')
+            sys.stderr.write(f'{str(err)}\n')
 
 
-class ElasticsearchSink:
+class ElasticsearchSink(_BaseSink):
     """Elasticsearch日志投递
     """
 
@@ -229,6 +371,8 @@ class ElasticsearchSink:
             delete_min_age: str = r'30d',
             **kwargs):
 
+        super().__init__(buffer_maxsize)
+
         self._elasticsearch = Elasticsearch(hosts, **kwargs)
 
         ElasticsearchDataStreamUtil(
@@ -239,127 +383,34 @@ class ElasticsearchSink:
 
         self._index = index
 
-        self._buffer = queue.Queue(buffer_maxsize)
-        self._closed = False
+    def _write_logs(self, logs):
 
-        self._task = Thread(target=self._do_task)
-        self._task.start()
-
-    def write(self, message):
-
-        if message.record[r'thread'].id == self._task.ident:
-            return
-
-        log_extra = message.record[r'extra']
-
-        if r'trace_id' not in log_extra and (trace_id := get_trace_id()) is not None:
-            log_extra[r'trace_id'] = trace_id
-
-        try:
-            self._buffer.put_nowait(message)
-        except queue.Full as _:
-            if message.record[r'level'].no > logging.INFO:
-                sys.stderr.write(str(message))
-
-    def close(self):
-
-        self._closed = True
-        self._task.join(10)
-
-    def _do_task(self):
-
-        while not self._closed:
-
-            messages = []
-
-            try:
-
-                messages.append(self._buffer.get(block=True, timeout=1))
-
-                for _ in range(min(self._buffer.qsize(), 1000)):
-                    messages.append(self._buffer.get_nowait())
-
-                if messages:
-                    es_helpers.bulk(
-                        self._elasticsearch,
-                        actions=[
-                            {
-                                r'_op_type': r'create',
-                                r'_index': self._index,
-                                r'extra': _msg.record[r'extra'],
-                                r'process': {
-                                    r'id': _msg.record[r'process'].id,
-                                    r'name': _msg.record[r'process'].name,
-                                },
-                                r'thread': {
-                                    r'id': _msg.record[r'thread'].id,
-                                    r'name': _msg.record[r'thread'].name,
-                                },
-                                r'level': {
-                                    r'no': _msg.record[r'level'].no,
-                                    r'name': _msg.record[r'level'].name,
-                                },
-                                r'module': f"{_msg.record[r'name']}:{_msg.record[r'function']}:{_msg.record[r'line']}",
-                                r'message': _msg.record[r'message'],
-                                r'@timestamp': int(_msg.record[r'time'].timestamp() * 1000),
-                            }
-                            for _msg in messages
-                        ]
-                    )
-
-            except queue.Empty as _:
-
-                pass
-
-            except Exception as _:
-
-                for _msg in messages:
-                    if _msg.record[r'level'].no > logging.INFO:
-                        sys.stderr.write(str(_msg))
-
-
-class QueuedFileSink(FileSink):
-    """日志文件队列
-    """
-
-    def __init__(self, path, *, buffer_maxsize=0xffff, **kwargs):
-
-        super().__init__(path, **kwargs)
-
-        self._buffer = queue.Queue(buffer_maxsize)
-
-        self._worker = Thread(target=self._queued_writer)
-        self._running = True
-
-        self._worker.start()
-
-    def write(self, message):
-
-        try:
-            self._buffer.put_nowait(message)
-        except queue.Full as _:
-            if message.record[r'level'].no > logging.INFO:
-                sys.stderr.write(str(message))
-
-    def stop(self):
-
-        self._running = False
-        self._worker.join(10)
-
-        super().stop()
-
-    def _queued_writer(self):
-
-        while self._running:
-
-            try:
-                super().write(
-                    self._buffer.get(block=True, timeout=1)
-                )
-            except queue.Empty as _:
-                pass
-            except Exception as err:
-                sys.stderr.write(str(err))
+        es_helpers.bulk(
+            self._elasticsearch,
+            actions=[
+                {
+                    r'_op_type': r'create',
+                    r'_index': self._index,
+                    r'extra': _log.record[r'extra'],
+                    r'process': {
+                        r'id': _log.record[r'process'].id,
+                        r'name': _log.record[r'process'].name,
+                    },
+                    r'thread': {
+                        r'id': _log.record[r'thread'].id,
+                        r'name': _log.record[r'thread'].name,
+                    },
+                    r'level': {
+                        r'no': _log.record[r'level'].no,
+                        r'name': _log.record[r'level'].name,
+                    },
+                    r'module': f"{_log.record[r'name']}:{_log.record[r'function']}:{_log.record[r'line']}",
+                    r'message': _log.record[r'message'],
+                    r'@timestamp': int(_log.record[r'time'].timestamp() * 1000),
+                }
+                for _log in logs
+            ]
+        )
 
 
 DEFAULT_LOG_FILE_NAME = r'runtime_{time}.log'
